@@ -1,4 +1,4 @@
-/* Copyright 2016--2018 The Tor Project
+/* Copyright 2016--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.metrics.stats.hidserv;
@@ -35,7 +35,7 @@ import java.util.TreeSet;
  * document files for later use. */
 public class Parser {
 
-  private static Logger log = LoggerFactory.getLogger(Parser.class);
+  private static final Logger logger = LoggerFactory.getLogger(Parser.class);
 
   /** File containing tuples of last-modified times and file names of
    * descriptor files parsed in the previous execution. */
@@ -56,6 +56,14 @@ public class Parser {
    * statistics. */
   private DocumentStore<ReportedHidServStats> reportedHidServStatsStore;
 
+  /** Document file containing previously parsed reported hidden-service
+   * statistics. */
+  private File reportedV3HidServStatsFile;
+
+  /** Document store for storing and retrieving reported hidden-service
+   * statistics. */
+  private DocumentStore<ReportedHidServStats> reportedV3HidServStatsStore;
+
   /** Directory containing document files with previously computed network
    * fractions. */
   private File computedNetworkFractionsDirectory;
@@ -69,6 +77,7 @@ public class Parser {
    * document stores. */
   public Parser(File[] inDirectories, File statusDirectory,
       DocumentStore<ReportedHidServStats> reportedHidServStatsStore,
+      DocumentStore<ReportedHidServStats> reportedV3HidServStatsStore,
       DocumentStore<ComputedNetworkFractions>
       computedNetworkFractionsStore) {
 
@@ -87,11 +96,14 @@ public class Parser {
     this.parseHistoryFile = new File(statusDirectory, "parse-history");
     this.reportedHidServStatsFile = new File(statusDirectory,
         "reported-hidserv-stats");
+    this.reportedV3HidServStatsFile = new File(statusDirectory,
+        "reported-v3hidserv-stats");
     this.computedNetworkFractionsDirectory =
         new File(statusDirectory, "computed-network-fractions");
 
     /* Store references to the provided document stores. */
     this.reportedHidServStatsStore = reportedHidServStatsStore;
+    this.reportedV3HidServStatsStore = reportedV3HidServStatsStore;
     this.computedNetworkFractionsStore = computedNetworkFractionsStore;
   }
 
@@ -111,12 +123,12 @@ public class Parser {
             String[] parts = line.split(" ", 2);
             excludedFiles.put(parts[1], Long.parseLong(parts[0]));
           } catch (NumberFormatException e) {
-            log.warn("Illegal line '{}' in parse history. Skipping line.", line,
-                e);
+            logger.warn("Illegal line '{}' in parse history. Skipping line.",
+                line, e);
           }
         }
       } catch (IOException e) {
-        log.warn("Could not read history file '{}'. Not "
+        logger.warn("Could not read history file '{}'. Not "
             + "excluding descriptors in this execution.",
             this.parseHistoryFile.getAbsolutePath(), e);
       }
@@ -147,12 +159,13 @@ public class Parser {
          * file, followed by its absolute path. */
         String absolutePath = e.getKey();
         long lastModifiedMillis = e.getValue();
-        bw.write(String.valueOf(lastModifiedMillis) + " " + absolutePath
+        bw.write(lastModifiedMillis + " " + absolutePath
             + "\n");
       }
     } catch (IOException e) {
-      log.warn("Could not write history file '{}'. Not excluding descriptors "
-          + "in next execution.", this.parseHistoryFile.getAbsolutePath(), e);
+      logger.warn("Could not write history file '{}'. Not excluding "
+          + "descriptors in next execution.",
+          this.parseHistoryFile.getAbsolutePath(), e);
     }
   }
 
@@ -162,6 +175,7 @@ public class Parser {
    * is easy.  But if this ever changes, e.g., when more and more
    * statistics are added, this may not scale.</p> */
   private Set<ReportedHidServStats> reportedHidServStats = new HashSet<>();
+  private Set<ReportedHidServStats> reportedV3HidServStats = new HashSet<>();
 
   /** Instructs the descriptor reader to parse descriptor files, and
    * handles the resulting parsed descriptors if they are either
@@ -184,6 +198,8 @@ public class Parser {
      * on. */
     this.reportedHidServStatsStore.store(
         this.reportedHidServStatsFile, this.reportedHidServStats);
+    this.reportedHidServStatsStore.store(
+        this.reportedV3HidServStatsFile, this.reportedV3HidServStats);
   }
 
   private static final String BIN_SIZE = "bin_size";
@@ -201,7 +217,9 @@ public class Parser {
 
     /* If the descriptor contained all expected hidserv-* lines, create a
      * new stats object and put it in the local map, so that it will later
-     * be written to a document file. */
+     * be written to a document file.
+     *
+     * Repeat for v3 */
     if (extraInfoDescriptor.getHidservStatsEndMillis() >= 0L
         && extraInfoDescriptor.getHidservStatsIntervalLength() >= 0L
         && extraInfoDescriptor.getHidservRendRelayedCells() != null
@@ -223,7 +241,6 @@ public class Parser {
           .getHidservDirOnionsSeen().longValue(), extraInfoDescriptor
           .getHidservDirOnionsSeenParameters().get(BIN_SIZE).longValue()));
       this.reportedHidServStats.add(reportedStats);
-
     /* If the descriptor contained some but not all hidserv-* lines, print
      * out a warning.  This case does not warrant any further action,
      * because relays can in theory write anything in their extra-info
@@ -231,9 +248,38 @@ public class Parser {
     } else if (extraInfoDescriptor.getHidservStatsEndMillis() >= 0L
         || extraInfoDescriptor.getHidservRendRelayedCells() != null
         || extraInfoDescriptor.getHidservDirOnionsSeen() != null) {
-      log.warn("Relay {} published incomplete hidserv-stats. Ignoring.",
+      logger.warn("Relay {} published incomplete hidserv-stats. Ignoring.",
           fingerprint);
     }
+
+    if (extraInfoDescriptor.getHidservV3StatsEndMillis() >= 0L
+        && extraInfoDescriptor.getHidservV3StatsIntervalLength() >= 0L
+        && extraInfoDescriptor.getHidservRendV3RelayedCells() != null
+        && extraInfoDescriptor.getHidservRendV3RelayedCellsParameters() != null
+        && extraInfoDescriptor.getHidservRendV3RelayedCellsParameters()
+        .containsKey(BIN_SIZE)
+    ) {
+      ReportedHidServStats reportedV3HidServStats = new ReportedHidServStats(
+          fingerprint, extraInfoDescriptor.getHidservV3StatsEndMillis());
+      reportedV3HidServStats.setStatsIntervalSeconds(extraInfoDescriptor
+          .getHidservV3StatsIntervalLength());
+      reportedV3HidServStats.setRendRelayedCells(
+          this.removeNoise(extraInfoDescriptor
+          .getHidservRendV3RelayedCells().longValue(), extraInfoDescriptor
+          .getHidservRendV3RelayedCellsParameters().get(BIN_SIZE).longValue()));
+      reportedV3HidServStats.setDirOnionsSeen(this.removeNoise(
+          extraInfoDescriptor
+          .getHidservDirV3OnionsSeen().longValue(), extraInfoDescriptor
+          .getHidservDirV3OnionsSeenParameters().get(BIN_SIZE).longValue()));
+      this.reportedV3HidServStats.add(reportedV3HidServStats);
+    } else if (extraInfoDescriptor.getHidservV3StatsEndMillis() >= 0L
+        || extraInfoDescriptor.getHidservRendV3RelayedCells() != null
+        || extraInfoDescriptor.getHidservDirV3OnionsSeen() != null) {
+      logger.warn("Relay {} published incomplete v3hidserv-stats. Ignoring.",
+          fingerprint);
+    }
+
+
   }
 
   /** Removes noise from a reported stats value by rounding to the nearest
@@ -252,7 +298,7 @@ public class Parser {
     SortedMap<String, Integer> bandwidthWeights =
         consensus.getBandwidthWeights();
     if (bandwidthWeights == null) {
-      log.warn("Consensus with valid-after time {} doesn't contain any Wxx "
+      logger.warn("Consensus with valid-after time {} doesn't contain any Wxx "
           + "weights. Skipping.",
           DateTimeHelper.format(consensus.getValidAfterMillis()));
       return;
@@ -264,7 +310,7 @@ public class Parser {
         new TreeSet<>(Arrays.asList("Wmg,Wmm,Wme,Wmd".split(",")));
     expectedWeightKeys.removeAll(bandwidthWeights.keySet());
     if (!expectedWeightKeys.isEmpty()) {
-      log.warn("Consensus with valid-after time {} doesn't contain expected "
+      logger.warn("Consensus with valid-after time {} doesn't contain expected "
           + "Wmx weights. Skipping.",
           DateTimeHelper.format(consensus.getValidAfterMillis()));
       return;
@@ -418,4 +464,3 @@ public class Parser {
         computedNetworkFractions);
   }
 }
-
